/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.android.core.proxy;

import android.content.Context;
import com.vektorsoft.demux.core.mva.DMXAdapter;
import com.vektorsoft.demux.core.mva.DMXDefaultAdapter;
import com.vektorsoft.demux.core.mva.IncompleteDeclarationException;
import com.vektorsoft.demux.core.proxy.ViewInvocationHandler;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * <p>
 * Implementation of invocation handler for Android views. It will use classes annotated with
 * {@link DMXViewDeclaration}, which must respect all rules described in {@link ViewInvocationHandler}.
 * In addition, classes which represent Android views must provide one of the following constructors:
 * </p>
 * <ul>
 *  <li>Default, no-argument constructor</li>
 *  <li>Constructor with single argument of type {@code android.content.Context }</li>
 *  <li>Constructor with two arguments of type {@code Context} and {@code DMXDefaultAdapter}</li>
 * </ul>
 * <p>
 *  Invocation handler will try to construct view object in the following way:
 * </p>
 * <ol>
 *  <li>
 * Search for constructor with signature {@code constr_name(Context, DMXDefaultAdapter)} and use it if found.
 * Clients should use this constructor if they need both {@code Context} and {@code DMXDefaultAdapter} type fields in
 * view operation.
 * </li>
 * <li>
 *  If the above is not found, search for constructor with signature {@code constr_name(Context)} and use 
 * it if found. Clients should provide this constructor if they need to use field of type {@code Context} in
 * view class.
 * </li>
 * <li>
 *  If none of the above are found, fall back to default, no-argument constructor.
 * </li>
 * </ol>
 *
 * @author Vladimir Djurovic
 */
public class AndroidViewInvocationHandler extends ViewInvocationHandler {
    
    
    
     /**
     * Create new instance. Parameter <code>clazz</code> is properly annotated view, with constructor
     * which takes an argument of type {@code android.content.Context}.
     * 
     * @param clazz view class
     * @param adapter adapter instance
     * @param context Android application context
     * @throws InstantiationException if class can not be instantiated
     * @throws IllegalAccessException if element can not be accessed
     * @throws NoSuchMethodException if class does not contain proper constructor
     * @throws InvocationTargetException if method cannot be invoked
     * @throws IncompleteDeclarationException if some of the required view annotations are missing
     */
    public AndroidViewInvocationHandler(Class clazz, DMXDefaultAdapter adapter, Context context) throws InstantiationException, IllegalAccessException, 
                                                                                            NoSuchMethodException, InvocationTargetException,
                                                                                            IncompleteDeclarationException{
        // check for appropriate constructor
        try {
            Constructor constr = clazz.getConstructor(Context.class, DMXAdapter.class);
            underlying = constr.newInstance(context, adapter);
        } catch(NoSuchMethodException nex){
            try {
                Constructor constr = clazz.getConstructor(Context.class);
                underlying = constr.newInstance(context);
            } catch(NoSuchMethodException nsex){
                underlying = clazz.newInstance();
            }
        }
       
        initialize(clazz, adapter);
    }

}
