/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/

package com.vektorsoft.demux.core.extension;

import com.vektorsoft.demux.core.mva.DMXViewManager;

/**
 * <p>
 * Declares methods for managing extensions and callbacks for the platform. General procedure for
 * managing extensions is the following:
 * </p>
 * <ul>
 *  <li>
 * extensions and callbacks can be registered in arbitrary order, so care must be taken that 
 * everything is properly initialized
 * </li>
 * <li>
 *  when {@code DMXExtendable} is registered, but no corresponding callback is yet available, it's
 * registration will be delayed until callback is available
 * </li>
 * <li>
 *  when callback is registered, corresponding extendable is initialized and registered
 * </li>
 * <li>
 *  when framework is started, all remaining instances of {@code DMXExtendable} are registered
 * </li>
 * </ul>
 *
 * @author Vladimir Djurovic
 */
public interface DMXExtensionManager {

    /**
     * <p>
     *  Registers a callback interface. This method will store callback in local 
     * cache, so it can be reused when appropriate.
     * </p>
     * 
     * @param callback callback interface
     */
    void registerCallback(DMXExtensionCallback callback);
    
    /**
     * <p>
     *  Registers specified instance of {@link DMXExtendable}, which should also be an
     * instance of {@link DMXController}.
     * </p>
     * 
     * @param extendable {@code DMXExtendable} to register
     */
    void registerExtendableController(DMXExtendable extendable);
    
    /**
     * <p>
     *  Registers specified instance of {@link DMXExtendable}, which should also be an
     * instance of {@link DMXView}.
     * </p>
     * 
     * @param view {@code DMXExtendable} to register
     * @param viewManager  view manager used to process registration
     */
    void registerExtendableView(DMXExtendable view, DMXViewManager viewManager);
}
