/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/
package com.vektorsoft.demux.core.mva;

import com.vektorsoft.demux.core.dlg.DMXDialogView;
import com.vektorsoft.demux.core.annotations.DMXViewDeclaration;
import com.vektorsoft.demux.core.extension.DMXExtensionCallback;
import com.vektorsoft.demux.core.extension.DMXExtensionType;
import com.vektorsoft.demux.core.proxy.ProxyUtils;
import com.vektorsoft.demux.core.proxy.ViewInvocationHandler;
import com.vektorsoft.demux.core.resources.LocaleChangeListener;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListMap;

/**
 * Base class for specific implementations of
 * <code>ViewManager</code>. Provides some common methods implementation that
 * can be reused by subclasses.
 */
public abstract class AbstractViewManager implements DMXViewManager {
    
    
     /** Adapter instance associated with this view manager. */
    protected DMXDefaultAdapter adapter;
    
    /** Set of listeners for locale change. */
    protected Set<LocaleChangeListener> localeListeners;
    
     /**
     * Contains all views registered with this manager. Map key is v iew name,
     * and value is the view itself.
     */
    protected final Map<String, DMXView> registeredViews;
    
    
    /** Empty set to be returned from methods. */
    private  final Set<String> emptySet;
    
    /** Views pending registration until their parent view is registered. */
    private final Map<String, Set<String>> deferredViews;
    
    /**
     * Contains map of data which are contained by the view. Map key is view ID,
     * and value is a set of data IDs.
     */
    private final Map<String, Set<String>> viewDataContents;
    
    /**
     * Maps data IDs to a set of view IDs that refer to this data. Map key is data ID,
     *  and value is a set of view IDs.
     */
    private final Map<String, Set<String>> dataToViewMapping;
    

    /**
     * Create new instance.
     */
    protected AbstractViewManager(){
        registeredViews = new ConcurrentSkipListMap<String, DMXView>();
        deferredViews = new ConcurrentSkipListMap<String, Set<String>>();
        viewDataContents = new ConcurrentSkipListMap<String, Set<String>>();
        dataToViewMapping = new ConcurrentSkipListMap<String, Set<String>>();
        localeListeners = Collections.synchronizedSet(new HashSet<LocaleChangeListener>());
        emptySet = new HashSet<String>();
    }
    
    /**
     * Called to complete registration of deferred views. Subclasses should override this method
     * to complete view registration, if required.
     * 
     * @param parentId registered parent view
     * @param childrenIds ids of dependent views
     */
    protected abstract void processDeferredViews(String parentId, Set<String> childrenIds);
    
    /**
     * Registers specified view for management by this view manager.
     * 
     * @param view view to register
     */
    @Override
    public  void registerView(DMXView view) {
        registeredViews.put(view.getViewId(), view);
        // register view data with the manager
        Set<String> viewDataIds = view.getViewDataIds();
        viewDataContents.put(view.getViewId(), viewDataIds);
        for(String entry : view.getViewDataIds()){
            
            // add this view to data-to-view mapping
            if(dataToViewMapping.containsKey(entry)){
                dataToViewMapping.get(entry).add(view.getViewId());
            } else {
                Set<String> viewIds = new HashSet<String>();
                viewIds.add(view.getViewId());
                dataToViewMapping.put(entry, viewIds);
            }
        }
        // check if parent view is registered and save this view to deffered if needed
        String parentId = view.getParentViewId();
        if(parentId != null && !parentId.isEmpty()){
            boolean registered = registeredViews.containsKey(parentId);
            if(!registered){
                if(deferredViews.containsKey(parentId)){
                    deferredViews.get(parentId).add(view.getViewId());
                } else {
                    Set<String> ids = new HashSet<String>();
                    ids.add(view.getViewId());
                    deferredViews.put(parentId, ids);
                }
            }
        }
        
    }
    
    /**
     * This method should be called from subclasses after initial view registration in order 
     * to correctly register deferred view.
     * 
     * @param view view to register
     */
    protected void afterInitialViewRegistration(DMXView view){
        // check if this view is needed for deffered views to complete registration
        String viewId = view.getViewId();
        if(deferredViews.containsKey(viewId)){
            processDeferredViews(viewId, deferredViews.get(viewId));
            deferredViews.remove(viewId);
        }
        // add this view to locale listeners, if needed
        if(view instanceof LocaleChangeListener){
            localeListeners.add((LocaleChangeListener)view);
        }
    }

    
    @Override
    public DMXView registerView(Class clazz) {
        DMXView view = null;
        if (clazz.isAnnotationPresent(DMXViewDeclaration.class)) {
            try {
                ViewInvocationHandler handler = new ViewInvocationHandler(clazz, adapter);
                List<Class> interfaces = new ArrayList<Class>();
                // check if this is dialog view, and set appropriate interface
                if(ProxyUtils.isDialogView(clazz)){
                    interfaces.add(DMXDialogView.class);
                } else {
                    interfaces.add(DMXView.class);
                }
                interfaces.addAll(ProxyUtils.getAllImplementedInterfaces(clazz));
                view = (DMXView)Proxy.newProxyInstance(this.getClass().getClassLoader(), interfaces.toArray(new Class[interfaces.size()]), handler);
                registerView(view);
                
            } catch (Exception ex) {
                ex.printStackTrace(System.err);
                throw new RuntimeException(ex);
            }

        }
        return view;
    }
    
    /**
     * Registers model data from view extension callback so it can be handled in
     * consistent fashion with the rest of the view data.
     * 
     * @param callback extension callback
     */
    protected void handleExtensionData(DMXExtensionCallback callback) {
        if (callback.getExtensionType() == DMXExtensionType.VIEW_EXTENSION) {
            Map<String, Object> data = callback.getCallbackData();
            if(data == null || data.isEmpty()){
                return;
            }
            String[] viewIds = callback.getMappings();
            for (String id : viewIds) {
                viewDataContents.put(id, data.keySet());
                for (String entry : data.keySet()) {
                    // add this callback to data-to-view mapping
                    if (dataToViewMapping.containsKey(entry)) {
                        dataToViewMapping.get(entry).add(id);
                    } else {
                        Set<String> ids = new HashSet<String>();
                        ids.add(id);
                        dataToViewMapping.put(entry, ids);
                    }
                }
            }
        }
    }

    
    /**
     * @see DMXViewManager#getView(java.lang.String) 
     * 
     * @param name view name
     * @return  view
     */
    @Override
    public DMXView getView(String name) {
        return registeredViews.get(name);
    }

    /**
     * @see DMXViewManager#getViewDataIds(java.lang.String) 
     * @param viewId view ID
     * @return data IDs registered for specified view
     */
    @Override
    public Set<String> getViewDataIds(String viewId) {
        if(viewDataContents.containsKey(viewId)){
            return viewDataContents.get(viewId);
        }
        return emptySet;
    }

    /**
     * {@inheritDoc }.
     */
    @Override
    public Set<String> getViewIdsForData(String dataId) {
        if(dataToViewMapping.containsKey(dataId)){
            return dataToViewMapping.get(dataId);
        }
        return emptySet;
    }
    
}
