/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.core.proxy;

import com.vektorsoft.demux.core.annotations.DMXControllerDeclaration;
import com.vektorsoft.demux.core.annotations.DMXModelData;
import com.vektorsoft.demux.core.extension.DMXExtendable;
import com.vektorsoft.demux.core.extension.DMXExtensionCallback;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 *  Invocation handler used for forwarding proxied methods for controllers.
 * </p>
 *
 * @author Vladimir Djurovic
 */
public class ControllerInvocationHandler implements InvocationHandler {
    
    /**
     * Extension callback used by this class.
     */
    protected DMXExtensionCallback callback;
    
    /** If <code>true</code>, indicates that underlying object implement
     * {@code DMXExtendable}.
     */
    protected boolean extendable = false;
    
    /** Underlying object for this handler. */
    private Object underlying;
    
    /** Controller mapping. */
    private String mapping;
    
    /** Next view ID for this handler. */
    private String nextViewId;
    
    /** Fields of underlying object declared as model data. */
    private Map<String, FieldAccessor> modelFields;
    
    /** Values of model data fields. */
    private Map<String, Object> modelData;
    
    /** Controller's {@code execute()} method. */
    private Method execMethod;
    
    
    /**
     * Creates new instance.
     * 
     * @param clazz controller class
     * @throws InstantiationException if class can not be instantiated
     * @throws IllegalAccessException if fields cannot be accessed
     * @throws InvocationTargetException if an error occurs during execution
     */
    public ControllerInvocationHandler(Class clazz) throws InstantiationException, IllegalAccessException, InvocationTargetException{
        DMXControllerDeclaration annotation = (DMXControllerDeclaration)clazz.getAnnotation(DMXControllerDeclaration.class);
        mapping = annotation.mapping();
        // if no mapping is specified, set object class name as mapping
        if(mapping.isEmpty()){
            mapping = clazz.getName();
        }
        nextViewId = annotation.nextViewId();
        
        modelFields = new HashMap<String, FieldAccessor>();
        modelData = new HashMap<String, Object>();
        execMethod = ProxyUtils.getControllerExecMethod(clazz);
        if(execMethod == null){
            throw new RuntimeException("Annotated controller must contain method with @DMXControllerDeclaration.Exec");
        }
        underlying = clazz.newInstance();
        // find model data fields and populate map
        DMXModelData dmxmodel = null;
        for(Field field : clazz.getDeclaredFields()){
            dmxmodel = field.getAnnotation(DMXModelData.class);
            if(dmxmodel != null){
                FieldAccessor fa = new FieldAccessor(field, underlying);
                String id = !dmxmodel.id().isEmpty() ? dmxmodel.id() : field.getName();
                modelFields.put(id, fa);
                modelData.put(id, fa.getFieldValue());
            }
        }
        for(Method m : clazz.getMethods()){
            if("getExtensionCallback".equals(m.getName())){
                callback = (DMXExtensionCallback)m.invoke(underlying);
                if(callback != null && callback.getCallbackData() != null && !callback.getCallbackData().isEmpty()){
                    modelData.putAll(callback.getCallbackData());
                }
                extendable = true;
                break;
            }
        }
    }

    @Override
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        if("getMapping".equals(method.getName())){
            return mapping;
        } else if("getNextViewId".equals(method.getName())){
            return nextViewId;
        } else if("getControllerDataMap".equals(method.getName())){
            return modelData;
        } else if("updateFromModel".equals(method.getName())){
            Map<String, Object> data = (Map<String, Object>)args[0];
            for(Map.Entry<String, Object> entry : data.entrySet()){
                if(modelFields.containsKey(entry.getKey())){
                    modelFields.get(entry.getKey()).setFieldValue(entry.getValue());
                }
                 if(callback != null){
                callback.setDataValuesFromModel(data);
            }
            }
        } else if("updateModel".equals(method.getName())){
            for(Map.Entry<String, FieldAccessor> entry : modelFields.entrySet()){
                modelData.put(entry.getKey(), entry.getValue().getFieldValue());
            }
            if(callback != null && callback.getCallbackData() != null && !callback.getCallbackData().isEmpty()){
                modelData.putAll(callback.getCallbackData());
            }
            return modelData;
        } else if("getControllerDataNames".equals(method.getName())){
            return modelData.keySet();
        } else if("execute".equals(method.getName())){
            execMethod.invoke(underlying, (Object[])args[0]);
        }else if(extendable && "setExtensionCallback".equals(method.getName())){
            ((DMXExtendable)underlying).setExtensionCallback((DMXExtensionCallback)args[0]);
            callback = (DMXExtensionCallback)args[0];
        } else if(extendable && "getExtensionCallback".equals(method.getName())){
            return callback;
        }  else {
            return method.invoke(underlying, args);
        }
        return null;
    }
    
    /**
     * Returns class name of the underlying object.
     * 
     * @return class name
     */
    public String getUnderlyingClassName(){
        return underlying.getClass().getName();
    }

}
