/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.core.resources.impl;

import com.vektorsoft.demux.core.extension.DMXExtensionCallback;
import com.vektorsoft.demux.core.resources.DMXResourceManager;
import com.vektorsoft.demux.core.resources.LocaleObservable;
import com.vektorsoft.demux.core.resources.ResourceHandlerType;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Observer;

/**
 * Default implementation of {@link DMXResourceManager}. This class provides platform-agnostic implementations
 * of methods from {@code DMXResourceManager}.
 *
 * @author Vladimir Djurovic
 */
public class DefaultResourceManager implements DMXResourceManager {
    
    /** List of locales supported by application. */
    protected List<Locale> supportedLocales;
    
    /** Application's current locale. */
    protected Locale currentLocale;
    
    /** Current resource handler. */
    protected DefaultResourceHandler resourceHandler;
   
    
    /** Observable object for notifying observers about locale change. */
    private LocaleObservable observable;
    
    /**
     * Creates new instance.
     */
    public DefaultResourceManager(){
        currentLocale = Locale.getDefault();
        supportedLocales = new ArrayList<Locale>();
        observable = new LocaleObservable();
        resourceHandler = new DefaultResourceHandler();
        resourceHandler.setCurrentLocale(currentLocale);
    }

    @Override
    public void setCurrentLocale(Locale locale) {
        if(locale == null){
            throw new IllegalArgumentException("Locale cannot be null");
        }
        this.currentLocale = locale;
        resourceHandler.setCurrentLocale(locale);
        resourceHandler.reloadBundles();
        observable.onLocaleChanged();
        observable.notifyObservers();
       
    }

    @Override
    public Locale getCurrentLocale() {
        return currentLocale;
    }

    @Override
    public void setSupportedLocales(Locale[] locales) {
        if(locales == null){
            throw new IllegalArgumentException("Supported locales cannot be null");
        }
        supportedLocales.clear();
        supportedLocales.addAll(Arrays.asList(locales));
    }

    @Override
    public Locale[] getSupportedLocales() {
        return supportedLocales.toArray(new Locale[supportedLocales.size()]);
    }

    @Override
    public void addSupportedLocale(Locale locale) {
        if(locale == null){
            throw new IllegalArgumentException("Supported locale cannot be null");
        }
        supportedLocales.add(locale);
    }

    @Override
    public void clearSupportedLocales() {
        supportedLocales.clear();
    }

    /**
     * Returns resource handler used for loading resources. For this implementation only valid
     * parameter is {@link ResourceHandlerType.RESOURCE_BUNDLE}. For any other parameter, an
     * {@code IllegalArgumentException} is thrown.
     * 
     * @param type resource handler type
     * @return resource handler
     */
    @Override
    public DefaultResourceHandler getResourceHandler(ResourceHandlerType type) {
        if(type == ResourceHandlerType.RESOURCE_BUNDLE){
            return  resourceHandler;
        } else {
            throw new IllegalArgumentException("Unsupported resource handler type. Use ResourceHandlerType.RESOURCE_BUNDLE.");
        }
        
    }

    
    @Override
    public void addLocaleChangeObserver(Observer observer) {
        observable.addObserver(observer);
    }

    
    @Override
    public void registerCallbackResources(DMXExtensionCallback callback) {
        resourceHandler.loadCallbackBundles(callback);
    }

}
