/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.desktop.jfx.app.gui;

import com.vektorsoft.demux.core.mva.DMXAdapter;
import com.vektorsoft.demux.core.mva.DMXEventRegistrationHandler;
import com.vektorsoft.demux.desktop.gui.DMXMenuService;
import com.vektorsoft.demux.desktop.jfx.gui.JFXEventHandler;
import javafx.application.Platform;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuBar;
import javafx.scene.control.MenuItem;
import javafx.scene.control.SeparatorMenuItem;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

/**
 * Default implementation of {@link DMXMenuService}.
 *
 * @author Vladimir Djurovic
 */
public class JFXMenuServiceImpl implements DMXMenuService<Menu, MenuItem> {
    
    /** Root layout for main window. */
    private final BorderPane root;
    
    /** Main menu bar. */
    private final MenuBar menuBar;
    
    /** Adapter instance. */
    private final DMXAdapter adapter;
    
    /**
     * Creates new instance.
     * 
     * @param stage main stage
     * @param adapter adapter
     */
    public JFXMenuServiceImpl(Stage stage, DMXAdapter adapter){
        this.adapter = adapter;
        root = (BorderPane)stage.getScene().getRoot();
        menuBar = new MenuBar();
        init();
    }
    
    /**
     * Initializes basic GUI layout.
     */
    private void init(){
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                ((VBox)root.getTop()).getChildren().add(menuBar);
            }
        });
    }

    

    @Override
    public void appendToMenuBar(final Menu menu) {
        
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                menuBar.getMenus().add(menu);
            }
        });
    }


    @Override
    public void addAfterMenu(final String menuId, final Menu menu) {
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                // if menuId is null, append to end
                if(menuId == null){
                    menuBar.getMenus().add(menu);
                    return;
                }
                // iterate through elements to find menu ID
                Menu curMenu;
                for(int i = 0;i < menuBar.getMenus().size();i++){
                    curMenu = menuBar.getMenus().get(i);
                    if(curMenu.getId() != null && menuId.equals(curMenu.getId())){
                        menuBar.getMenus().add(i + 1, menu);
                        return;
                    }
                }
                // if no required ID is found, append to end
                menuBar.getMenus().add(menu);
            }
        });
    }

    @Override
    public void insertAtIndex(final int index, final Menu menu) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                if (menuBar.getMenus().size() > index) {
                    menuBar.getMenus().add(index, menu);
                } else {
                    menuBar.getMenus().add(menu);
                }
            }
        });
    }

    @Override
    public void addMenuItemToMenu(final String menuId, final MenuItem menuItem) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                // find menu with specified ID
                for (Menu menu : menuBar.getMenus()) {
                    if (menu.getId() != null && menu.getId().equals(menuId)) {
                        menu.getItems().add(menuItem);
                        break;
                    }
                }
            }
        });

    }

    @Override
    public void insertMenuItem(final String menuId, final int index, final MenuItem menuItem) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                // find menu with specified ID
                for (Menu menu : menuBar.getMenus()) {
                    if (menu.getId() != null && menu.getId().equals(menuId)) {
                        if(menu.getItems().size() > index){
                            menu.getItems().add(index, menuItem);
                        } else {
                            menu.getItems().add(menuItem);
                        }
                        break;
                    }
                }
            }
        });
    }
    
    

    @Override
    public void addSubmenuToMenu(final String menuId, final Menu subMenu) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                // find menu with specified ID
                for (Menu menu : menuBar.getMenus()) {
                    if (menu.getId() != null && menu.getId().equals(menuId)) {
                        menu.getItems().add(subMenu);
                        break;
                    }
                }
            }
        });

    }

    @Override
    public void insertSeparator(final String menuId, final int index) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                // find menu with specified ID
                for (Menu menu : menuBar.getMenus()) {
                    if (menu.getId() != null && menu.getId().equals(menuId)) {
                        if(menu.getItems().size() > index){
                            menu.getItems().add(index, new SeparatorMenuItem());
                        } 
                        break;
                    }
                }
            }
        });
    }

    @Override
    public void registerControllerForAction(final String menuItemId, final String controllerId, final Object... params) {
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                MenuItem menuItem = JavaFXUtils.findMenuItemById(menuBar, menuItemId);
                if (menuItem != null) {
                    menuItem.setOnAction(new MenuActionControllerInvoker(adapter, controllerId, params));
                }

            }
        });

    }

    @Override
    public void registerEventHandlerForMenuItem(final String menuItemId, final DMXEventRegistrationHandler handler) {
        if(handler instanceof JFXEventHandler){
            Platform.runLater(new Runnable() {

            @Override
            public void run() {
                JFXEventHandler jhandler = (JFXEventHandler)handler;
                MenuItem menuItem = JavaFXUtils.findMenuItemById(menuBar, menuItemId);
                System.out.println("Found menu item " + menuItem);
                if (menuItem != null) {
                    menuItem.addEventHandler(jhandler.getEventType(), jhandler);
                }

            }
        });
        } else {
            throw new IllegalArgumentException("Invalid event handler type, required com.vektorsoft.demux.desktop.app.JFXEventHandler");
        }
         
    }
    
    
    

}
