/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.desktop.jfx.gui;

import com.vektorsoft.demux.core.mva.DMXAdapter;
import com.vektorsoft.demux.core.mva.DMXEventRegistrationHandler;
import java.util.ArrayList;
import java.util.List;
import javafx.beans.value.ChangeListener;

/**
 * <p>
 *  Base class for registering property change listeners for Java FX GUI components. Subclasses must specify which property
 * type they want to listen to and what is the method for getting property values.
 * </p>
 * <p>
 * Simple implementation of this class could look like the following:
 *  <code>
 *      <pre>
 *          class MyClass extends JFXPropertyChangeListener &lt; BooleanProperty &gt; {
 *              public MyClass(DMXAdapter adapter){
                    super(adapter, "selectedProperty");
                    addComponentId("someId");
                }
 *          }
 *      </pre>
 * </code>
 * </p>
 * <p>
 *  In this sample, listener will handle {@code BooleanProperty}, and the method for getting the value of this 
 * property is <code>selectedProperty</code>, as specified in constructor. This method is invoked using reflection.
 * </p>
 *
 * @param <T> type of the observable property
 * @author Vladimir Djurovic
 */
public abstract class JFXPropertyChangeHandler<T> implements DMXEventRegistrationHandler<ChangeListener<T>>, ChangeListener<T> {
    
    /**
     * Adapter instance.
     */
    protected DMXAdapter adapter;
    
    /**
     * Name of the method for getting property value.
     */
    protected String property;
    
    /** List of component IDs for which this listener is registered. */
    private final List<String> componentIds;
    

    /**
     * Creates new instance.
     * 
     * @param adapter adapter instance
     * @param property name of the method for getting property value
     */
    protected JFXPropertyChangeHandler(DMXAdapter adapter, String property) {
        this.property = property;
        this.adapter = adapter;
        componentIds = new ArrayList<>();
    }
    
    /**
     * Add component ID to the list.
     * 
     * @param id component ID
     */
    public void addComponentId(String id){
        componentIds.add(id);
    }
    
    /**
     * Set component IDs to the list.
     * 
     * @param ids component IDs
     */
    public void setComponentIds(String... ids){
        for(String id : ids){
            componentIds.add(id);
        }
    }

    @Override
    public String[] getComponentIds() {
        return componentIds.toArray(new String[componentIds.size()]);
    }

    @Override
    public ChangeListener<T> getEventHandler() {
        return this;
    }

    /**
     * Returns the name of the property getter method.
     * 
     * @return method name
     */
    public String getProperty() {
        return property;
    }


}
